from enum import Enum


class BackgroundTaskType(str, Enum):
    ENDPOINT_REPLAY = "endpoint.replay"
    ENDPOINT_RECOVER = "endpoint.recover"
    APPLICATION_STATS = "application.stats"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar

import attr

T = TypeVar("T", bound="EndpointStats")


@attr.s(auto_attribs=True)
class EndpointStats:
    """
    Attributes:
        fail (int):
        pending (int):
        sending (int):
        success (int):
    """

    fail: int
    pending: int
    sending: int
    success: int
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        fail = self.fail
        pending = self.pending
        sending = self.sending
        success = self.success

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "fail": fail,
                "pending": pending,
                "sending": sending,
                "success": success,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        dict_copy = src_dict.copy()
        fail = dict_copy.pop("fail")

        pending = dict_copy.pop("pending")

        sending = dict_copy.pop("sending")

        success = dict_copy.pop("success")

        endpoint_stats = cls(
            fail=fail,
            pending=pending,
            sending=sending,
            success=success,
        )

        endpoint_stats.additional_properties = dict_copy
        return endpoint_stats

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.message_attempt_failed_data import MessageAttemptFailedData
from ..types import UNSET, Unset

T = TypeVar("T", bound="MessageAttemptFailingEventData")


@attr.s(auto_attribs=True)
class MessageAttemptFailingEventData:
    """Sent when a message delivery has failed (all of the retry attempts have been exhausted) as a
    "message.attempt.exhausted" type or after it's failed four times as a "message.attempt.failing" event.

        Attributes:
            app_id (str): The app's ID Example: app_1srOrx2ZWZBpBUvZwXKQmoEYga2.
            endpoint_id (str): The ep's ID Example: ep_1srOrx2ZWZBpBUvZwXKQmoEYga2.
            last_attempt (MessageAttemptFailedData):
            msg_id (str): The msg's ID Example: msg_1srOrx2ZWZBpBUvZwXKQmoEYga2.
            app_uid (Union[Unset, None, str]): The app's UID Example: unique-app-identifier.
            msg_event_id (Union[Unset, None, str]): The msg's UID Example: unique-msg-identifier.
    """

    app_id: str
    endpoint_id: str
    last_attempt: MessageAttemptFailedData
    msg_id: str
    app_uid: Union[Unset, None, str] = UNSET
    msg_event_id: Union[Unset, None, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        app_id = self.app_id
        endpoint_id = self.endpoint_id
        last_attempt = self.last_attempt.to_dict()

        msg_id = self.msg_id
        app_uid = self.app_uid
        msg_event_id = self.msg_event_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "appId": app_id,
                "endpointId": endpoint_id,
                "lastAttempt": last_attempt,
                "msgId": msg_id,
            }
        )
        if app_uid is not UNSET:
            field_dict["appUid"] = app_uid
        if msg_event_id is not UNSET:
            field_dict["msgEventId"] = msg_event_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        dict_copy = src_dict.copy()
        app_id = dict_copy.pop("appId")

        endpoint_id = dict_copy.pop("endpointId")

        last_attempt = MessageAttemptFailedData.from_dict(dict_copy.pop("lastAttempt"))

        msg_id = dict_copy.pop("msgId")

        app_uid = dict_copy.pop("appUid", UNSET)

        msg_event_id = dict_copy.pop("msgEventId", UNSET)

        message_attempt_failing_event_data = cls(
            app_id=app_id,
            endpoint_id=endpoint_id,
            last_attempt=last_attempt,
            msg_id=msg_id,
            app_uid=app_uid,
            msg_event_id=msg_event_id,
        )

        message_attempt_failing_event_data.additional_properties = dict_copy
        return message_attempt_failing_event_data

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import random
from time import sleep
from typing import Any, Dict

import httpx

from ...client import AuthenticatedClient
from ...models.application_in import ApplicationIn
from ...models.application_out import ApplicationOut
from ...models.http_error import HttpError
from ...models.http_validation_error import HTTPValidationError


def _get_kwargs(
    app_id: str,
    *,
    client: AuthenticatedClient,
    json_body: ApplicationIn,
) -> Dict[str, Any]:
    url = "{}/api/v1/app/{app_id}/".format(client.base_url, app_id=app_id)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    json_json_body = json_body.to_dict()

    return {
        "method": "put",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "json": json_json_body,
    }


def _parse_response(*, response: httpx.Response) -> ApplicationOut:
    if response.status_code == 401:
        raise HttpError(response.json(), response.status_code)
    if response.status_code == 403:
        raise HttpError(response.json(), response.status_code)
    if response.status_code == 404:
        raise HttpError(response.json(), response.status_code)
    if response.status_code == 409:
        raise HttpError(response.json(), response.status_code)
    if response.status_code == 422:
        raise HTTPValidationError(response.json(), response.status_code)
    if response.status_code == 429:
        raise HttpError(response.json(), response.status_code)
    response_200 = ApplicationOut.from_dict(response.json())

    return response_200


sleep_time = 0.05
num_retries = 3


def sync_detailed(
    app_id: str,
    *,
    client: AuthenticatedClient,
    json_body: ApplicationIn,
) -> ApplicationOut:
    """Update Application

     Update an application.

    Args:
        app_id (str): The app's ID or UID Example: unique-app-identifier.
        json_body (ApplicationIn):

    Returns:
        ApplicationOut
    """

    kwargs = _get_kwargs(
        app_id=app_id,
        client=client,
        json_body=json_body,
    )

    kwargs["headers"] = {"svix-req-id": f"{random.getrandbits(32)}", **kwargs["headers"]}

    retry_count = 0
    for retry in range(num_retries):
        response = httpx.request(
            verify=client.verify_ssl,
            **kwargs,
        )
        if response.status_code >= 500 and retry < num_retries:
            retry_count += 1
            kwargs["headers"]["svix-retry-count"] = str(retry_count)
            sleep(sleep_time)
            sleep_time * 2
        else:
            break

    return _parse_response(response=response)


def sync(
    app_id: str,
    *,
    client: AuthenticatedClient,
    json_body: ApplicationIn,
) -> ApplicationOut:
    """Update Application

     Update an application.

    Args:
        app_id (str): The app's ID or UID Example: unique-app-identifier.
        json_body (ApplicationIn):

    Returns:
        ApplicationOut
    """

    return sync_detailed(
        app_id=app_id,
        client=client,
        json_body=json_body,
    )


async def asyncio_detailed(
    app_id: str,
    *,
    client: AuthenticatedClient,
    json_body: ApplicationIn,
) -> ApplicationOut:
    """Update Application

     Update an application.

    Args:
        app_id (str): The app's ID or UID Example: unique-app-identifier.
        json_body (ApplicationIn):

    Returns:
        ApplicationOut
    """

    kwargs = _get_kwargs(
        app_id=app_id,
        client=client,
        json_body=json_body,
    )

    kwargs["headers"] = {"svix-req-id": f"{random.getrandbits(32)}", **kwargs["headers"]}

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        retry_count = 0
        for retry in range(num_retries):
            response = await _client.request(**kwargs)
            if response.status_code >= 500 and retry < num_retries:
                retry_count += 1
                kwargs["headers"]["svix-retry-count"] = str(retry_count)
                sleep(sleep_time)
                sleep_time * 2
            else:
                break

    return _parse_response(response=response)


async def asyncio(
    app_id: str,
    *,
    client: AuthenticatedClient,
    json_body: ApplicationIn,
) -> ApplicationOut:
    """Update Application

     Update an application.

    Args:
        app_id (str): The app's ID or UID Example: unique-app-identifier.
        json_body (ApplicationIn):

    Returns:
        ApplicationOut
    """

    return await asyncio_detailed(
        app_id=app_id,
        client=client,
        json_body=json_body,
    )



from typing import Any, Dict, List, Type, TypeVar

import attr

from ..models.endpoint_disabled_event_data import EndpointDisabledEventData
from ..models.endpoint_disabled_event_type import EndpointDisabledEventType

T = TypeVar("T", bound="EndpointDisabledEvent")


@attr.s(auto_attribs=True)
class EndpointDisabledEvent:
    """Sent when an endpoint has been automatically disabled after continuous failures.

    Attributes:
        data (EndpointDisabledEventData): Sent when an endpoint has been automatically disabled after continuous
            failures.
        type (EndpointDisabledEventType):  Default: EndpointDisabledEventType.ENDPOINT_DISABLED.
    """

    data: EndpointDisabledEventData
    type: EndpointDisabledEventType = EndpointDisabledEventType.ENDPOINT_DISABLED
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        data = self.data.to_dict()

        type = self.type.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "data": data,
                "type": type,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        dict_copy = src_dict.copy()
        data = EndpointDisabledEventData.from_dict(dict_copy.pop("data"))

        type = EndpointDisabledEventType(dict_copy.pop("type"))

        endpoint_disabled_event = cls(
            data=data,
            type=type,
        )

        endpoint_disabled_event.additional_properties = dict_copy
        return endpoint_disabled_event

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Dict, TypeVar

T = TypeVar("T", bound="HttpError")


class HttpError(Exception):
    status_code: int

    def __init__(self, response: Dict[str, str], status_code: int):
        self.status_code = status_code
        super(HttpError, self).__init__(response)



import datetime
from typing import Any, Dict, List, Type, TypeVar

import attr
from dateutil.parser import isoparse

from ..models.message_attempt_trigger_type import MessageAttemptTriggerType
from ..models.message_status import MessageStatus

T = TypeVar("T", bound="MessageAttemptOut")


@attr.s(auto_attribs=True)
class MessageAttemptOut:
    """
    Attributes:
        endpoint_id (str): The ep's ID Example: ep_1srOrx2ZWZBpBUvZwXKQmoEYga2.
        id (str): The attempt's ID Example: atmpt_1srOrx2ZWZBpBUvZwXKQmoEYga2.
        msg_id (str): The msg's ID Example: msg_1srOrx2ZWZBpBUvZwXKQmoEYga2.
        response (str):  Example: {}.
        response_status_code (int):  Example: 200.
        status (MessageStatus): The sending status of the message:
            - Success = 0
            - Pending = 1
            - Fail = 2
            - Sending = 3
        timestamp (datetime.datetime):
        trigger_type (MessageAttemptTriggerType): The reason an attempt was made:
            - Scheduled = 0
            - Manual = 1
        url (str):  Example: https://example.com/webhook/.
    """

    endpoint_id: str
    id: str
    msg_id: str
    response: str
    response_status_code: int
    status: MessageStatus
    timestamp: datetime.datetime
    trigger_type: MessageAttemptTriggerType
    url: str
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        endpoint_id = self.endpoint_id
        id = self.id
        msg_id = self.msg_id
        response = self.response
        response_status_code = self.response_status_code
        status = self.status.value

        timestamp = self.timestamp.isoformat()

        trigger_type = self.trigger_type.value

        url = self.url

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "endpointId": endpoint_id,
                "id": id,
                "msgId": msg_id,
                "response": response,
                "responseStatusCode": response_status_code,
                "status": status,
                "timestamp": timestamp,
                "triggerType": trigger_type,
                "url": url,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        dict_copy = src_dict.copy()
        endpoint_id = dict_copy.pop("endpointId")

        id = dict_copy.pop("id")

        msg_id = dict_copy.pop("msgId")

        response = dict_copy.pop("response")

        response_status_code = dict_copy.pop("responseStatusCode")

        status = MessageStatus(dict_copy.pop("status"))

        timestamp = isoparse(dict_copy.pop("timestamp"))

        trigger_type = MessageAttemptTriggerType(dict_copy.pop("triggerType"))

        url = dict_copy.pop("url")

        message_attempt_out = cls(
            endpoint_id=endpoint_id,
            id=id,
            msg_id=msg_id,
            response=response,
            response_status_code=response_status_code,
            status=status,
            timestamp=timestamp,
            trigger_type=trigger_type,
            url=url,
        )

        message_attempt_out.additional_properties = dict_copy
        return message_attempt_out

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import Any, Dict, List, Type, TypeVar

import attr
from dateutil.parser import isoparse

T = TypeVar("T", bound="IntegrationOut")


@attr.s(auto_attribs=True)
class IntegrationOut:
    """
    Attributes:
        created_at (datetime.datetime):
        id (str): The integ's ID Example: integ_1srOrx2ZWZBpBUvZwXKQmoEYga2.
        name (str):  Example: Example Integration.
        updated_at (datetime.datetime):
    """

    created_at: datetime.datetime
    id: str
    name: str
    updated_at: datetime.datetime
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        created_at = self.created_at.isoformat()

        id = self.id
        name = self.name
        updated_at = self.updated_at.isoformat()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "createdAt": created_at,
                "id": id,
                "name": name,
                "updatedAt": updated_at,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        dict_copy = src_dict.copy()
        created_at = isoparse(dict_copy.pop("createdAt"))

        id = dict_copy.pop("id")

        name = dict_copy.pop("name")

        updated_at = isoparse(dict_copy.pop("updatedAt"))

        integration_out = cls(
            created_at=created_at,
            id=id,
            name=name,
            updated_at=updated_at,
        )

        integration_out.additional_properties = dict_copy
        return integration_out

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



""" Contains all the data models used in inputs/outputs """

from .app_portal_access_in import AppPortalAccessIn
from .app_portal_access_out import AppPortalAccessOut
from .app_usage_stats_in import AppUsageStatsIn
from .app_usage_stats_out import AppUsageStatsOut
from .application_in import ApplicationIn
from .application_in_metadata import ApplicationInMetadata
from .application_out import ApplicationOut
from .application_out_metadata import ApplicationOutMetadata
from .application_patch import ApplicationPatch
from .application_patch_metadata import ApplicationPatchMetadata
from .application_stats import ApplicationStats
from .application_token_expire_in import ApplicationTokenExpireIn
from .attempt_statistics_data import AttemptStatisticsData
from .attempt_statistics_response import AttemptStatisticsResponse
from .background_task_data import BackgroundTaskData
from .background_task_out import BackgroundTaskOut
from .background_task_status import BackgroundTaskStatus
from .background_task_type import BackgroundTaskType
from .border_radius_config import BorderRadiusConfig
from .border_radius_enum import BorderRadiusEnum
from .custom_color_palette import CustomColorPalette
from .custom_theme_override import CustomThemeOverride
from .dashboard_access_out import DashboardAccessOut
from .endpoint_created_event import EndpointCreatedEvent
from .endpoint_created_event_data import EndpointCreatedEventData
from .endpoint_created_event_type import EndpointCreatedEventType
from .endpoint_deleted_event import EndpointDeletedEvent
from .endpoint_deleted_event_data import EndpointDeletedEventData
from .endpoint_deleted_event_type import EndpointDeletedEventType
from .endpoint_disabled_event import EndpointDisabledEvent
from .endpoint_disabled_event_data import EndpointDisabledEventData
from .endpoint_disabled_event_type import EndpointDisabledEventType
from .endpoint_headers_in import EndpointHeadersIn
from .endpoint_headers_in_headers import EndpointHeadersInHeaders
from .endpoint_headers_out import EndpointHeadersOut
from .endpoint_headers_out_headers import EndpointHeadersOutHeaders
from .endpoint_headers_patch_in import EndpointHeadersPatchIn
from .endpoint_headers_patch_in_headers import EndpointHeadersPatchInHeaders
from .endpoint_in import EndpointIn
from .endpoint_in_metadata import EndpointInMetadata
from .endpoint_message_out import EndpointMessageOut
from .endpoint_message_out_payload import EndpointMessageOutPayload
from .endpoint_out import EndpointOut
from .endpoint_out_metadata import EndpointOutMetadata
from .endpoint_patch import EndpointPatch
from .endpoint_patch_metadata import EndpointPatchMetadata
from .endpoint_secret_out import EndpointSecretOut
from .endpoint_secret_rotate_in import EndpointSecretRotateIn
from .endpoint_stats import EndpointStats
from .endpoint_transformation_in import EndpointTransformationIn
from .endpoint_transformation_out import EndpointTransformationOut
from .endpoint_transformation_simulate_in import EndpointTransformationSimulateIn
from .endpoint_transformation_simulate_in_payload import EndpointTransformationSimulateInPayload
from .endpoint_transformation_simulate_out import EndpointTransformationSimulateOut
from .endpoint_transformation_simulate_out_payload import EndpointTransformationSimulateOutPayload
from .endpoint_update import EndpointUpdate
from .endpoint_update_metadata import EndpointUpdateMetadata
from .endpoint_updated_event import EndpointUpdatedEvent
from .endpoint_updated_event_data import EndpointUpdatedEventData
from .endpoint_updated_event_type import EndpointUpdatedEventType
from .environment_in import EnvironmentIn
from .environment_out import EnvironmentOut
from .environment_settings_out import EnvironmentSettingsOut
from .event_example_in import EventExampleIn
from .event_type_example_out import EventTypeExampleOut
from .event_type_example_out_example import EventTypeExampleOutExample
from .event_type_example_out_example_additional_property import EventTypeExampleOutExampleAdditionalProperty
from .event_type_in import EventTypeIn
from .event_type_in_schemas import EventTypeInSchemas
from .event_type_in_schemas_additional_property import EventTypeInSchemasAdditionalProperty
from .event_type_out import EventTypeOut
from .event_type_out_schemas import EventTypeOutSchemas
from .event_type_out_schemas_additional_property import EventTypeOutSchemasAdditionalProperty
from .event_type_patch import EventTypePatch
from .event_type_patch_schemas import EventTypePatchSchemas
from .event_type_patch_schemas_additional_property import EventTypePatchSchemasAdditionalProperty
from .event_type_schema_in import EventTypeSchemaIn
from .event_type_schema_in_schema import EventTypeSchemaInSchema
from .event_type_update import EventTypeUpdate
from .event_type_update_schemas import EventTypeUpdateSchemas
from .event_type_update_schemas_additional_property import EventTypeUpdateSchemasAdditionalProperty
from .export_environment_in import ExportEnvironmentIn
from .font_size_config import FontSizeConfig
from .http_error import HttpError
from .http_validation_error import HTTPValidationError
from .integration_in import IntegrationIn
from .integration_key_out import IntegrationKeyOut
from .integration_out import IntegrationOut
from .integration_update import IntegrationUpdate
from .list_response_application_out import ListResponseApplicationOut
from .list_response_application_stats import ListResponseApplicationStats
from .list_response_background_task_out import ListResponseBackgroundTaskOut
from .list_response_endpoint_message_out import ListResponseEndpointMessageOut
from .list_response_endpoint_out import ListResponseEndpointOut
from .list_response_event_type_out import ListResponseEventTypeOut
from .list_response_integration_out import ListResponseIntegrationOut
from .list_response_message_attempt_endpoint_out import ListResponseMessageAttemptEndpointOut
from .list_response_message_attempt_out import ListResponseMessageAttemptOut
from .list_response_message_endpoint_out import ListResponseMessageEndpointOut
from .list_response_message_out import ListResponseMessageOut
from .message_attempt_endpoint_out import MessageAttemptEndpointOut
from .message_attempt_exhausted_event import MessageAttemptExhaustedEvent
from .message_attempt_exhausted_event_data import MessageAttemptExhaustedEventData
from .message_attempt_exhausted_event_type import MessageAttemptExhaustedEventType
from .message_attempt_failed_data import MessageAttemptFailedData
from .message_attempt_failing_event import MessageAttemptFailingEvent
from .message_attempt_failing_event_data import MessageAttemptFailingEventData
from .message_attempt_failing_event_type import MessageAttemptFailingEventType
from .message_attempt_headers_out import MessageAttemptHeadersOut
from .message_attempt_headers_out_sent_headers import MessageAttemptHeadersOutSentHeaders
from .message_attempt_out import MessageAttemptOut
from .message_attempt_trigger_type import MessageAttemptTriggerType
from .message_endpoint_out import MessageEndpointOut
from .message_in import MessageIn
from .message_in_payload import MessageInPayload
from .message_out import MessageOut
from .message_out_payload import MessageOutPayload
from .message_status import MessageStatus
from .ordering import Ordering
from .recover_in import RecoverIn
from .recover_out import RecoverOut
from .replay_in import ReplayIn
from .replay_out import ReplayOut
from .settings_in import SettingsIn
from .settings_out import SettingsOut
from .statistics_period import StatisticsPeriod
from .status_code_class import StatusCodeClass
from .transformation_http_method import TransformationHttpMethod
from .validation_error import ValidationError



from typing import Any, Dict, List, Type, TypeVar, cast

import attr

from ..models.message_attempt_headers_out_sent_headers import MessageAttemptHeadersOutSentHeaders

T = TypeVar("T", bound="MessageAttemptHeadersOut")


@attr.s(auto_attribs=True)
class MessageAttemptHeadersOut:
    """
    Attributes:
        sensitive (List[str]):
        sent_headers (MessageAttemptHeadersOutSentHeaders):
    """

    sensitive: List[str]
    sent_headers: MessageAttemptHeadersOutSentHeaders
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        sensitive = self.sensitive

        sent_headers = self.sent_headers

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "sensitive": sensitive,
                "sentHeaders": sent_headers,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        dict_copy = src_dict.copy()
        sensitive = cast(List[str], dict_copy.pop("sensitive"))

        sent_headers = dict_copy.pop("sentHeaders")

        message_attempt_headers_out = cls(
            sensitive=sensitive,
            sent_headers=sent_headers,
        )

        message_attempt_headers_out.additional_properties = dict_copy
        return message_attempt_headers_out

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import datetime
from typing import Any, Dict, List, Type, TypeVar, Union

import attr
from dateutil.parser import isoparse

from ..types import UNSET, Unset

T = TypeVar("T", bound="RecoverIn")


@attr.s(auto_attribs=True)
class RecoverIn:
    """
    Attributes:
        since (datetime.datetime):
        until (Union[Unset, None, datetime.datetime]):
    """

    since: datetime.datetime
    until: Union[Unset, None, datetime.datetime] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        since = self.since.isoformat()

        until: Union[Unset, None, str] = UNSET
        if not isinstance(self.until, Unset):
            until = self.until.isoformat() if self.until else None

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "since": since,
            }
        )
        if until is not UNSET:
            field_dict["until"] = until

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        dict_copy = src_dict.copy()
        since = isoparse(dict_copy.pop("since"))

        _until = dict_copy.pop("until", UNSET)
        until: Union[Unset, None, datetime.datetime]
        if _until is None:
            until = None
        elif isinstance(_until, Unset):
            until = UNSET
        else:
            until = isoparse(_until)

        recover_in = cls(
            since=since,
            until=until,
        )

        recover_in.additional_properties = dict_copy
        return recover_in

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar

import attr

from ..models.message_attempt_exhausted_event_data import MessageAttemptExhaustedEventData
from ..models.message_attempt_exhausted_event_type import MessageAttemptExhaustedEventType

T = TypeVar("T", bound="MessageAttemptExhaustedEvent")


@attr.s(auto_attribs=True)
class MessageAttemptExhaustedEvent:
    """Sent when a message delivery has failed (all of the retry attempts have been exhausted).

    Attributes:
        data (MessageAttemptExhaustedEventData): Sent when a message delivery has failed (all of the retry attempts have
            been exhausted) as a "message.attempt.exhausted" type or after it's failed four times as a
            "message.attempt.failing" event.
        type (MessageAttemptExhaustedEventType):  Default: MessageAttemptExhaustedEventType.MESSAGE_ATTEMPT_EXHAUSTED.
    """

    data: MessageAttemptExhaustedEventData
    type: MessageAttemptExhaustedEventType = MessageAttemptExhaustedEventType.MESSAGE_ATTEMPT_EXHAUSTED
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        data = self.data.to_dict()

        type = self.type.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "data": data,
                "type": type,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        dict_copy = src_dict.copy()
        data = MessageAttemptExhaustedEventData.from_dict(dict_copy.pop("data"))

        type = MessageAttemptExhaustedEventType(dict_copy.pop("type"))

        message_attempt_exhausted_event = cls(
            data=data,
            type=type,
        )

        message_attempt_exhausted_event.additional_properties = dict_copy
        return message_attempt_exhausted_event

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.border_radius_config import BorderRadiusConfig
from ..models.font_size_config import FontSizeConfig
from ..types import UNSET, Unset

T = TypeVar("T", bound="CustomThemeOverride")


@attr.s(auto_attribs=True)
class CustomThemeOverride:
    """
    Attributes:
        border_radius (Union[Unset, BorderRadiusConfig]):
        font_size (Union[Unset, FontSizeConfig]):
    """

    border_radius: Union[Unset, BorderRadiusConfig] = UNSET
    font_size: Union[Unset, FontSizeConfig] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        border_radius: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.border_radius, Unset):
            border_radius = self.border_radius.to_dict()

        font_size: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.font_size, Unset):
            font_size = self.font_size.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if border_radius is not UNSET:
            field_dict["borderRadius"] = border_radius
        if font_size is not UNSET:
            field_dict["fontSize"] = font_size

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        dict_copy = src_dict.copy()
        _border_radius = dict_copy.pop("borderRadius", UNSET)
        border_radius: Union[Unset, BorderRadiusConfig]
        if isinstance(_border_radius, Unset):
            border_radius = UNSET
        else:
            border_radius = BorderRadiusConfig.from_dict(_border_radius)

        _font_size = dict_copy.pop("fontSize", UNSET)
        font_size: Union[Unset, FontSizeConfig]
        if isinstance(_font_size, Unset):
            font_size = UNSET
        else:
            font_size = FontSizeConfig.from_dict(_font_size)

        custom_theme_override = cls(
            border_radius=border_radius,
            font_size=font_size,
        )

        custom_theme_override.additional_properties = dict_copy
        return custom_theme_override

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



import random
from time import sleep
from typing import Any, Dict, Union

import httpx

from ...client import AuthenticatedClient
from ...models.event_type_in import EventTypeIn
from ...models.event_type_out import EventTypeOut
from ...models.http_error import HttpError
from ...models.http_validation_error import HTTPValidationError
from ...types import UNSET, Unset


def _get_kwargs(
    *,
    client: AuthenticatedClient,
    json_body: EventTypeIn,
    idempotency_key: Union[Unset, str] = UNSET,
) -> Dict[str, Any]:
    url = "{}/api/v1/event-type/".format(client.base_url)

    headers: Dict[str, str] = client.get_headers()
    cookies: Dict[str, Any] = client.get_cookies()

    if not isinstance(idempotency_key, Unset):
        headers["idempotency-key"] = idempotency_key

    json_json_body = json_body.to_dict()

    return {
        "method": "post",
        "url": url,
        "headers": headers,
        "cookies": cookies,
        "timeout": client.get_timeout(),
        "json": json_json_body,
    }


def _parse_response(*, response: httpx.Response) -> EventTypeOut:
    if response.status_code == 401:
        raise HttpError(response.json(), response.status_code)
    if response.status_code == 403:
        raise HttpError(response.json(), response.status_code)
    if response.status_code == 404:
        raise HttpError(response.json(), response.status_code)
    if response.status_code == 409:
        raise HttpError(response.json(), response.status_code)
    if response.status_code == 422:
        raise HTTPValidationError(response.json(), response.status_code)
    if response.status_code == 429:
        raise HttpError(response.json(), response.status_code)
    response_201 = EventTypeOut.from_dict(response.json())

    return response_201


sleep_time = 0.05
num_retries = 3


def sync_detailed(
    *,
    client: AuthenticatedClient,
    json_body: EventTypeIn,
    idempotency_key: Union[Unset, str] = UNSET,
) -> EventTypeOut:
    """Create Event Type

     Create new or unarchive existing event type.

    Unarchiving an event type will allow endpoints to filter on it and messages to be sent with it.
    Endpoints filtering on the event type before archival will continue to filter on it.
    This operation does not preserve the description and schemas.

    Args:
        idempotency_key (Union[Unset, str]):
        json_body (EventTypeIn):

    Returns:
        EventTypeOut
    """

    kwargs = _get_kwargs(
        client=client,
        json_body=json_body,
        idempotency_key=idempotency_key,
    )

    kwargs["headers"] = {"svix-req-id": f"{random.getrandbits(32)}", **kwargs["headers"]}

    retry_count = 0
    for retry in range(num_retries):
        response = httpx.request(
            verify=client.verify_ssl,
            **kwargs,
        )
        if response.status_code >= 500 and retry < num_retries:
            retry_count += 1
            kwargs["headers"]["svix-retry-count"] = str(retry_count)
            sleep(sleep_time)
            sleep_time * 2
        else:
            break

    return _parse_response(response=response)


def sync(
    *,
    client: AuthenticatedClient,
    json_body: EventTypeIn,
    idempotency_key: Union[Unset, str] = UNSET,
) -> EventTypeOut:
    """Create Event Type

     Create new or unarchive existing event type.

    Unarchiving an event type will allow endpoints to filter on it and messages to be sent with it.
    Endpoints filtering on the event type before archival will continue to filter on it.
    This operation does not preserve the description and schemas.

    Args:
        idempotency_key (Union[Unset, str]):
        json_body (EventTypeIn):

    Returns:
        EventTypeOut
    """

    return sync_detailed(
        client=client,
        json_body=json_body,
        idempotency_key=idempotency_key,
    )


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    json_body: EventTypeIn,
    idempotency_key: Union[Unset, str] = UNSET,
) -> EventTypeOut:
    """Create Event Type

     Create new or unarchive existing event type.

    Unarchiving an event type will allow endpoints to filter on it and messages to be sent with it.
    Endpoints filtering on the event type before archival will continue to filter on it.
    This operation does not preserve the description and schemas.

    Args:
        idempotency_key (Union[Unset, str]):
        json_body (EventTypeIn):

    Returns:
        EventTypeOut
    """

    kwargs = _get_kwargs(
        client=client,
        json_body=json_body,
        idempotency_key=idempotency_key,
    )

    kwargs["headers"] = {"svix-req-id": f"{random.getrandbits(32)}", **kwargs["headers"]}

    async with httpx.AsyncClient(verify=client.verify_ssl) as _client:
        retry_count = 0
        for retry in range(num_retries):
            response = await _client.request(**kwargs)
            if response.status_code >= 500 and retry < num_retries:
                retry_count += 1
                kwargs["headers"]["svix-retry-count"] = str(retry_count)
                sleep(sleep_time)
                sleep_time * 2
            else:
                break

    return _parse_response(response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    json_body: EventTypeIn,
    idempotency_key: Union[Unset, str] = UNSET,
) -> EventTypeOut:
    """Create Event Type

     Create new or unarchive existing event type.

    Unarchiving an event type will allow endpoints to filter on it and messages to be sent with it.
    Endpoints filtering on the event type before archival will continue to filter on it.
    This operation does not preserve the description and schemas.

    Args:
        idempotency_key (Union[Unset, str]):
        json_body (EventTypeIn):

    Returns:
        EventTypeOut
    """

    return await asyncio_detailed(
        client=client,
        json_body=json_body,
        idempotency_key=idempotency_key,
    )



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..models.custom_color_palette import CustomColorPalette
from ..models.custom_theme_override import CustomThemeOverride
from ..types import UNSET, Unset

T = TypeVar("T", bound="EnvironmentSettingsOut")


@attr.s(auto_attribs=True)
class EnvironmentSettingsOut:
    """
    Attributes:
        color_palette_dark (Union[Unset, CustomColorPalette]):
        color_palette_light (Union[Unset, CustomColorPalette]):
        custom_color (Union[Unset, None, str]):
        custom_font_family (Union[Unset, None, str]):  Example: Open Sans.
        custom_logo_url (Union[Unset, None, str]):
        custom_theme_override (Union[Unset, CustomThemeOverride]):
        enable_channels (Union[Unset, bool]):
        enable_integration_management (Union[Unset, bool]):
        enable_transformations (Union[Unset, bool]):
    """

    color_palette_dark: Union[Unset, CustomColorPalette] = UNSET
    color_palette_light: Union[Unset, CustomColorPalette] = UNSET
    custom_color: Union[Unset, None, str] = UNSET
    custom_font_family: Union[Unset, None, str] = UNSET
    custom_logo_url: Union[Unset, None, str] = UNSET
    custom_theme_override: Union[Unset, CustomThemeOverride] = UNSET
    enable_channels: Union[Unset, bool] = False
    enable_integration_management: Union[Unset, bool] = False
    enable_transformations: Union[Unset, bool] = False
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        color_palette_dark: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.color_palette_dark, Unset):
            color_palette_dark = self.color_palette_dark.to_dict()

        color_palette_light: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.color_palette_light, Unset):
            color_palette_light = self.color_palette_light.to_dict()

        custom_color = self.custom_color
        custom_font_family = self.custom_font_family
        custom_logo_url = self.custom_logo_url
        custom_theme_override: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.custom_theme_override, Unset):
            custom_theme_override = self.custom_theme_override.to_dict()

        enable_channels = self.enable_channels
        enable_integration_management = self.enable_integration_management
        enable_transformations = self.enable_transformations

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if color_palette_dark is not UNSET:
            field_dict["colorPaletteDark"] = color_palette_dark
        if color_palette_light is not UNSET:
            field_dict["colorPaletteLight"] = color_palette_light
        if custom_color is not UNSET:
            field_dict["customColor"] = custom_color
        if custom_font_family is not UNSET:
            field_dict["customFontFamily"] = custom_font_family
        if custom_logo_url is not UNSET:
            field_dict["customLogoUrl"] = custom_logo_url
        if custom_theme_override is not UNSET:
            field_dict["customThemeOverride"] = custom_theme_override
        if enable_channels is not UNSET:
            field_dict["enableChannels"] = enable_channels
        if enable_integration_management is not UNSET:
            field_dict["enableIntegrationManagement"] = enable_integration_management
        if enable_transformations is not UNSET:
            field_dict["enableTransformations"] = enable_transformations

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        dict_copy = src_dict.copy()
        _color_palette_dark = dict_copy.pop("colorPaletteDark", UNSET)
        color_palette_dark: Union[Unset, CustomColorPalette]
        if isinstance(_color_palette_dark, Unset):
            color_palette_dark = UNSET
        else:
            color_palette_dark = CustomColorPalette.from_dict(_color_palette_dark)

        _color_palette_light = dict_copy.pop("colorPaletteLight", UNSET)
        color_palette_light: Union[Unset, CustomColorPalette]
        if isinstance(_color_palette_light, Unset):
            color_palette_light = UNSET
        else:
            color_palette_light = CustomColorPalette.from_dict(_color_palette_light)

        custom_color = dict_copy.pop("customColor", UNSET)

        custom_font_family = dict_copy.pop("customFontFamily", UNSET)

        custom_logo_url = dict_copy.pop("customLogoUrl", UNSET)

        _custom_theme_override = dict_copy.pop("customThemeOverride", UNSET)
        custom_theme_override: Union[Unset, CustomThemeOverride]
        if isinstance(_custom_theme_override, Unset):
            custom_theme_override = UNSET
        else:
            custom_theme_override = CustomThemeOverride.from_dict(_custom_theme_override)

        enable_channels = dict_copy.pop("enableChannels", UNSET)

        enable_integration_management = dict_copy.pop("enableIntegrationManagement", UNSET)

        enable_transformations = dict_copy.pop("enableTransformations", UNSET)

        environment_settings_out = cls(
            color_palette_dark=color_palette_dark,
            color_palette_light=color_palette_light,
            custom_color=custom_color,
            custom_font_family=custom_font_family,
            custom_logo_url=custom_logo_url,
            custom_theme_override=custom_theme_override,
            enable_channels=enable_channels,
            enable_integration_management=enable_integration_management,
            enable_transformations=enable_transformations,
        )

        environment_settings_out.additional_properties = dict_copy
        return environment_settings_out

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="AttemptStatisticsData")


@attr.s(auto_attribs=True)
class AttemptStatisticsData:
    """
    Attributes:
        failure_count (Union[Unset, None, List[int]]):
        success_count (Union[Unset, None, List[int]]):
    """

    failure_count: Union[Unset, None, List[int]] = UNSET
    success_count: Union[Unset, None, List[int]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        failure_count: Union[Unset, None, List[int]] = UNSET
        if not isinstance(self.failure_count, Unset):
            if self.failure_count is None:
                failure_count = None
            else:
                failure_count = self.failure_count

        success_count: Union[Unset, None, List[int]] = UNSET
        if not isinstance(self.success_count, Unset):
            if self.success_count is None:
                success_count = None
            else:
                success_count = self.success_count

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if failure_count is not UNSET:
            field_dict["failureCount"] = failure_count
        if success_count is not UNSET:
            field_dict["successCount"] = success_count

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        dict_copy = src_dict.copy()
        failure_count = cast(List[int], dict_copy.pop("failureCount", UNSET))

        success_count = cast(List[int], dict_copy.pop("successCount", UNSET))

        attempt_statistics_data = cls(
            failure_count=failure_count,
            success_count=success_count,
        )

        attempt_statistics_data.additional_properties = dict_copy
        return attempt_statistics_data

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
